#include "send-dialog.h"

#include <limits.h>

#include <QtDebug>

#include <TelepathyQt4/PendingChannelRequest>


SendDialog::SendDialog(QWidget *parent) :
    QProgressDialog(parent),
    fileOffsetShift(0)
{
}

void
SendDialog::setPendingChannelRequest(Tp::PendingChannelRequest *pendingOp)
{
    connect(pendingOp, SIGNAL(channelRequestCreated(Tp::ChannelRequestPtr)),
            SLOT(onChannelRequestCreated(Tp::ChannelRequestPtr)));
    connect(pendingOp, SIGNAL(finished(Tp::PendingOperation*)),
            SLOT(onPendingChannelRequestFinished(Tp::PendingOperation*)));

    // FIXME: pendingOp should be cancelled when this dialog is cancelled.
    // Unfortunately Tp::PendingChannelRequest::cancel is not a slot.

    setLabelText(tr("Initializing..."));
}

void SendDialog::setFile(QIODevice *in)
{
    if (!file.isNull()) {
        qCritical() << "cannot replace the file for the send channel";
        return;
    }

    file = in;
}

void
SendDialog::onPendingChannelRequestFinished(Tp::PendingOperation *pendingOp)
{
    if (pendingOp->isError()) {
        qDebug() << "file transfer channel request creation failed:"
                 << pendingOp->errorName() << "-" << pendingOp->errorMessage();
        setLabelText(tr("Error: ") + pendingOp->errorMessage());
    }
}

void
SendDialog::onChannelRequestCreated(Tp::ChannelRequestPtr request)
{
    qDebug() << "file transfer channel request created";
    channelRequest = request;

    connect(channelRequest.data(),
            SIGNAL(succeeded(const Tp::ChannelPtr&)),
            SLOT(onChannelRequestSucceeded(const Tp::ChannelPtr&)));
    connect(channelRequest.data(),
            SIGNAL(failed(const QString&, const QString&)),
            SLOT(onChannelRequestFailed(const QString&, const QString&)));
}

void
SendDialog::onChannelRequestSucceeded(const Tp::ChannelPtr &channel)
{
    qDebug() << "file transfer channel request succeeded";
    channelRequest.reset();

    sendChannel = Tp::OutgoingFileTransferChannelPtr::staticCast(channel);

    if (file.isNull()) {
        qWarning() << "the file object has been destroyed before transfer could start";
        sendChannel->cancel();
        return;
    }

    connect(sendChannel.data(),
            SIGNAL(stateChanged(Tp::FileTransferState, Tp::FileTransferStateChangeReason)),
            SLOT(onTransferStateChanged(Tp::FileTransferState, Tp::FileTransferStateChangeReason)));
    connect(sendChannel.data(),
            SIGNAL(initialOffsetDefined(qulonglong)),
            SLOT(onInitialOffsetDefined(qulonglong)));
    connect(sendChannel.data(),
            SIGNAL(transferredBytesChanged(qulonglong)),
            SLOT(onTransferredBytesChanged(qulonglong)));

    // TODO: add a handler to gracefully cancel and close the channel
    // when the dialog is cancelled

    sendChannel->provideFile(file.data());

    updateTransferStateMessage(sendChannel->state());

    // Calibrate the scale for the progress value
    qulonglong sizeScaled = sendChannel->size();
    while (sizeScaled > (qulonglong) INT_MAX) {
        sizeScaled >>= 1;
        ++fileOffsetShift;
    }

    setMaximum(sizeScaled);
}

void
SendDialog::onChannelRequestFailed(const QString &errorName,
                                   const QString &errorMessage)
{
    qDebug() << "File transfer channel request failed:" << errorName << "-" << errorMessage;

    setLabelText(tr("Error: ") + errorMessage);
}

void
SendDialog::onTransferStateChanged(Tp::FileTransferState state,
                                   Tp::FileTransferStateChangeReason reason)
{
    // TODO: some state changes mean it is over, may want to close the window
    updateTransferStateMessage(state, reason);
}

void
SendDialog::onInitialOffsetDefined(qulonglong offset)
{
    setValue(scaleFileOffset(offset));
}

void
SendDialog::onTransferredBytesChanged(qulonglong count)
{
    setValue(scaleFileOffset(count));
}

void
SendDialog::updateTransferStateMessage(Tp::FileTransferState state,
                                       Tp::FileTransferStateChangeReason reason)
{
    QString message;
    switch (state) {
    case Tp::FileTransferStatePending:
        message = tr("Waiting");
        break;
    case Tp::FileTransferStateAccepted:
        message = tr("Accepted");
        break;
    case Tp::FileTransferStateOpen:
        message = tr("Transferring");
        break;
    case Tp::FileTransferStateCompleted:
        message = tr("Completed");
        break;
    case Tp::FileTransferStateCancelled:
        // TODO: regard the reason for a more detailed message
        message = tr("Cancelled");
        break;
    default:
        qDebug() << "unexpected file transfer state" << (unsigned int) state;
    }
}
